@@ -1,11 +1,76 @@
 package AxKit::App::TABOO;
 
+our $VERSION = '0.39_1';
+
 use 5.7.3;
 use strict;
 use warnings;
 
-our $VERSION = '0.18_14';
+use Session;
+use Apache;
+use Apache::Cookie;
+use Apache::Request;
+use AxKit;
 
+sub session_config {
+  my $r = shift;
+  my $prefix = $r->dir_config( 'TABOOPrefix') || 'TABOO';
+  my %session_config = (
+			Store     => $r->dir_config( $prefix . 'DataStore' ) || 'DB_File',
+			Lock      => $r->dir_config( $prefix . 'Lock' ) || 'Null',
+			Generate  => $r->dir_config( $prefix . 'Generate' ) || 'MD5',
+			Serialize => $r->dir_config( $prefix . 'Serialize' ) || 'Storable'
+		       );
+   
+  # When using Postgres, a different default is needed.  
+  if ($session_config{'Store'} eq 'Postgres') {
+    $session_config{'Commit'} = 1;
+    $session_config{'Serialize'} = $r->dir_config( $prefix . 'Serialize' ) || 'Base64'
+  }
+
+  
+  # Load session-type specific parameters, comma-separated, name => value pairs
+  foreach my $arg ( split( /\s*,\s*/, $r->dir_config($prefix . 'Args'))) {
+    my ($key, $value) = split( /\s*=>\s*/, $arg );
+    $session_config{$key} = $value;
+  }
+
+  return %session_config;
+}
+
+sub session {
+  my $r = shift;
+  my $req = Apache::Request->instance($r);
+  my $vid = $req->param('VID');
+  unless ($vid) { # Then look in the cookie
+    my $cookies = Apache::Cookie->fetch;
+    my $tmp = $cookies->{'VID'};
+    return undef unless defined($tmp);
+    $vid = $tmp->value;
+    return undef unless ($vid); # No session key
+  }
+  AxKit::Debug(9, "User's visitor ID: $vid");
+  return new Session $vid, session_config($r);
+}
+    
+
+sub authlevel {
+  my $session = shift;
+  if (defined($session)) {
+    return $session->get('authlevel');
+  } else {
+    return 0;
+  }
+}
+
+sub loggedin {
+  my $session = shift;
+  if (defined($session)) {
+    return $session->get('loggedin');
+  } else {
+    return 'guest';
+  }
+}
 
 1;
 __END__
@@ -17,15 +82,14 @@ AxKit::App::TABOO - Object Oriented Publishing Framework for AxKit
 
 =head1 INTRODUCTION
 
-There is no code in this file. It may be some day, but for now, it is
-a placeholder, but one where it is convenient to say what this is, and
-my design philosophy for it. 
-
 AxKit::App::TABOO is a object oriented approach to creating a
 publishing system on the top of AxKit, the XML Application Server. 
 The two O's thus stands for Object Oriented, AB for AxKit-Based. 
 I don't know what the T stands for yet, suggestions are welcome! 
 
+This file holds only a few session and authentication functions, that
+I didn't quite knew where to put. This may change.
+
 
 =head1 DESIGN PHILOSOPHY
 
@@ -92,21 +156,27 @@ above objects at a time, something that's often necessary. It also
 provides some containment of complexity, taking worries off of your
 head!
 
-Then, there are two AxKit Providers:
-L<AxKit::App::TABOO::Provider::News> and
-L<AxKit::App::TABOO::Provider::NewsList>, that makes use of the above
-subclasses, especially Story and Comment in the first case and the
-Stories in the second. The first provider creates a page containing an
+Then, there are three working AxKit Providers:
+L<AxKit::App::TABOO::Provider::News>,
+L<AxKit::App::TABOO::Provider::NewsList> and
+L<AxKit::App::TABOO::Provider::Classification>, that makes use of the
+above subclasses, especially Story and Comment in the first case, the
+Stories in the second and a combination of those, Categories and
+Articles in the last. The first provider creates a page containing an
 editor-reviewed story and user-submitted comments. The second makes
-lists of stories. By simply manipulating the URI in easy-to-understand
-ways, you can load just the story, view the comments, separately, in a
-list or as a thread, or get simple lists or good overviews of stories.
+lists of stories. The third creates lists of content based on
+categories. By simply manipulating the URI in easy-to-understand ways,
+you can load just the story, view the comments, separately, in a list
+or as a thread, or get simple lists or good overviews of stories, or
+lists of articles and stories in a category.
 
-Currently, it supplies four Taglibs,
+Currently, it supplies six Taglibs,
 L<User|AxKit::App::TABOO::XSP::User>,
 L<Story|AxKit::App::TABOO::XSP::Story>,
-L<Comment|AxKit::App::TABOO::XSP::Comment> and
-L<Category|AxKit::App::TABOO::XSP::Category>. These taglibs provide
+L<Comment|AxKit::App::TABOO::XSP::Comment>,
+L<Comment|AxKit::App::TABOO::XSP::Articles>,
+L<Category|AxKit::App::TABOO::XSP::Category> and
+L<Category|AxKit::App::TABOO::XSP::Language>. These taglibs provide
 several tags that you may use interface with the Data objects.
 
 There is quite a lot of XSP and XSLT now that allows you to enter and edit
@@ -128,6 +198,11 @@ take all strings of text from a separate XML file and insert them in
 the final product. This makes it easy to provide many translations
 with TABOO, allthough a real multilingual site is not yet supported.
 
+Pionered in 0.2 is the first code to manage more static articles,
+typically background articles with rich metadata. Content can be
+displayed, and that's pretty robust. Entering articles also works, but
+they cannot be edited yet.
+
 =head1 CONFIGURATION EXAMPLE
 
 The following is most of the author's AxKit configuration, and should
@@ -147,7 +222,6 @@ separate username and password on a databse, and use different
 databases for different virtual hosts. A combination of C<DBI_DSN>,
 C<PGUSER> and C<PGPASSWORD> environment variables will achieve this.
 
-
   # \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
   # The below directives are common for all virtual hosts, if you run such
 
@@ -199,26 +273,35 @@ C<PGUSER> and C<PGPASSWORD> environment variables will achieve this.
   AxAddPlugin Apache::AxKit::Plugin::Param::Expr
   PerlAddVar AxParamExpr neg.lang '"nb"'
 
+  AxAddPlugin AxKit::App::TABOO::AddXSLParams::Session
 
-  # Authentication
-  PerlModule Apache::AxKit::Plugin::BasicAuth
-  AxAddPlugin Apache::AxKit::Plugin::BasicSession
-  AxAddPlugin Apache::AxKit::Plugin::AddXSLParams::BasicSession
 
   # TABOO XSPs
   AxAddXSPTaglib AxKit::App::TABOO::XSP::User
   AxAddXSPTaglib AxKit::App::TABOO::XSP::Story
   AxAddXSPTaglib AxKit::App::TABOO::XSP::Category
   AxAddXSPTaglib AxKit::App::TABOO::XSP::Comment
+#  AxAddXSPTaglib AxKit::App::TABOO::XSP::Article
+  AxAddXSPTaglib AxKit::App::TABOO::XSP::Language
+
+
+
 
   # Other XSPs
-  AxAddXSPTaglib AxKit::XSP::BasicAuth
-  AxAddXSPTaglib AxKit::XSP::BasicSession
+
   AxAddXSPTaglib AxKit::XSP::QueryParam
   AxAddXSPTaglib AxKit::XSP::Sendmail
 
   PerlAddVar AxParamExpr cats.prefix '"/cats/"'
 
+  PerlModule AxKit::App::TABOO::Handler::Login
+  <Location /login>
+     SetHandler perl-script
+     PerlHandler AxKit::App::TABOO::Handler::Login
+     PerlSendHeader On
+  </Location>
+
+
   <Location /cats/>
   	PerlHandler AxKit
         AxContentProvider AxKit::App::TABOO::Provider::Classification
@@ -284,23 +367,16 @@ C<PGUSER> and C<PGPASSWORD> environment variables will achieve this.
   # Aliases, rather than files have a full filesystem path. 
   # That's rather evil...
   Alias /news/submit /var/www/news/submit.xsp 
-  Alias /login /var/www/login.xsp
   Alias /categories/submit /var/www/categories/submit.xsp 
 
+  Alias /articles/submit /var/www/articles/submit.xsp 
+  Alias /articles/edit /var/www/articles/edit.xsp 
+
 
   # Authentication and authorization stuff
-  <Location />
-      AuthType Apache::AxKit::Plugin::BasicAuth
-      AuthName TABOODemo
-  </Location>
 
-  PerlSetVar TABOODemoURIToken SID
-  PerlSetVar TABOODemoDataStore DB_File
-  PerlSetVar TABOODemoArgs      "FileName => /tmp/taboodemo-session"
-  PerlSetVar TABOODemoLoginScript /login.xsp
-  # If you like to keep people logged in over a longer time, you may 
-  # extend their session, eg:
-  PerlSetVar TABOODemoCookieExpires +14d
+  PerlSetVar TABOODataStore DB_File
+  PerlSetVar TABOOArgs      "FileName => /tmp/taboodemo-session"
 
   # /////////////////////////////
 
@@ -321,9 +397,10 @@ multiple users can post news stories, and comment them. That's
 something that has been done before, of course, but not within the
 design goals stated above.
 
-From now on most of the attention will be on the static articles from
-now on. A lot of new code is in the present distro, but it has quite a
-lot of issues.
+The Article code needs work still, manipulate articles better, but I
+have actually put what I have in production, and it works!
+
+Also, I need some kind of gallery code soonish.
 
 Finally note that things that are there are B<not stable>! Names may
 change, parameters may be different, and I may decide to do things
@@ -340,7 +417,7 @@ the rest of framework. TABOO will make a great webshop platform...!
 
 =head1 SUPPORT
 
-There is now a taboo-dev mailing list that can be subscribed to at 
+There is now a taboo-dev mailing list that can be subscribed to at
 http://lists.kjernsmo.net/mailman/listinfo/taboo-dev
 
 =head1 BUGS
@@ -348,6 +425,16 @@ http://lists.kjernsmo.net/mailman/listinfo/taboo-dev
 There are surely some... Please report any you find through CPAN RT:
 http://rt.cpan.org/NoAuth/Bugs.html?Dist=AxKit-App-TABOO .
 
+=head1 SUBVERSION REPOSITORY
+
+TABOO is currently maintained in a Subversion repository. The trunk
+can be checked out anonymously using e.g.:
+
+  svn checkout http://svn.kjernsmo.net/TABOO/trunk TABOO  
+
+It is also available using the svn schema and svn+ssh for committers.
+
+
 =head1 AUTHOR
 
 Kjetil Kjernsmo, E<lt>kjetilk@cpan.orgE<gt>
@@ -358,7 +445,7 @@ L<AxKit>, L<AxKit::App::TABOO::Data>, L<AxKit::App::TABOO::Provider::News>.
 
 =head1 COPYRIGHT AND LICENCE
 
-Copyright (c) 2003-2005 Kjetil Kjernsmo. This program is free
+Copyright (c) 2003-2006 Kjetil Kjernsmo. This program is free
 software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
 
