@@ -11,7 +11,7 @@ use Carp;
 # module.
 
 
-our $VERSION = '0.19_02';
+our $VERSION = '0.4';
 
 =head1 NAME
 
@@ -21,7 +21,7 @@ AxKit::App::TABOO::Provider::Classification - Provider for classifiying things i
 
 In the Apache config:
 
-  <Location /category/>
+  <Location /cats/>
   	PerlHandler AxKit
         AxContentProvider AxKit::App::TABOO::Provider::Classification
   </Location>
@@ -49,24 +49,6 @@ doesn't deal with comments.
 The rest of the documentation has yet to be written, but as one can
 guess, it may share some things with the NewsList Provider.
 
-=head1 CONFIGURATION DIRECTIVES
-
-=over
-
-=item TABOOListDefaultRecords
-
-The maximum number of stories TABOO will retrieve from the data store
-if the user gives no other instructions in the URI (see below). It is
-recommended that you set this to some reasonable value.
-
-=item TABOOListMaxRecords
-
-The maximum number of stories TABOO will retrieve from the data store
-in any case. If the user requests more than this number, a 403
-Forbidden error will be returned. It is highly recommended that you
-set this to a value you think your server can handle.
-
-=back
 
 =cut
 
@@ -84,6 +66,7 @@ use Apache::AxKit::Exception;
 use Apache::AxKit::Provider;
 
 use AxKit;
+use AxKit::App::TABOO;
 use AxKit::App::TABOO::Data::Story;
 use AxKit::App::TABOO::Data::Plurals::Stories;
 use AxKit::App::TABOO::Data::Category;
@@ -91,8 +74,6 @@ use AxKit::App::TABOO::Data::Plurals::Categories;
 use AxKit::App::TABOO::Data::Article;
 use AxKit::App::TABOO::Data::Plurals::Articles;
 
-use Apache::AxKit::Plugin::BasicSession;
-
 
 sub init {
   my $self = shift;
@@ -101,15 +82,14 @@ sub init {
   
   AxKit::Debug(10, "[Classification] Request object: " . $r->as_string);
   AxKit::Debug(8, "[Classification] Provider using URI " . $r->uri);
-  
-  $self->{number} = $r->dir_config('TABOOListDefaultRecords');
-  $self->{maxrecords} = $r->dir_config('TABOOListMaxRecords');
 
   $self->{uri} = $r->uri;
   my @uri = split('/', $r->uri);
   splice(@uri, 0, 2); # The first part is just a keyword,
   $self->{cats} = \@uri;
 
+  $self->{session} = AxKit::App::TABOO::session($r);
+
   AxKit::Debug(9, "[Classification] Data parsed in init: " . Dumper($self));
   return $self;
 }
@@ -133,7 +113,7 @@ sub process {
     $self->{stories} = AxKit::App::TABOO::Data::Plurals::Stories->new();
     unless ($self->{articles}->incat(@{$self->{cats}})) {
       $self->{articles} = undef;
-      unless ($self->{stories}->incat(@{$self->{cats}})) { 
+      unless ($self->{stories}->exists(catname => @{$self->{cats}})) { 
 	$self->{exists} = 0;
       }
     }
@@ -153,9 +133,8 @@ sub exists {
 
 
 sub key {
-  no warnings; # To avoid warning when no credential_0 exists
   my $self = shift;
-  return $self->{uri} . "/" . $Apache::AxKit::Plugin::BasicSession::session{credential_0};
+  return $self->{uri} . "/" . AxKit::App::TABOO::loggedin($self->{session});
 }
 
 
@@ -178,11 +157,12 @@ sub get_dom {
 					     return_code => 404,
 					     -text => "Category " . $self->{catnotfound} . " was not found");
     }
-    warn "ANT: " . Dumper($self);
+#    warn "ANT: " . Dumper($self);
+    my $authlevel = AxKit::App::TABOO::authlevel($self->{session});
     if (scalar(@{$self->{cats}}) == 1) {
       my %limit;
-      if (! defined($Apache::AxKit::Plugin::BasicSession::session{authlevel})
-	  || ($Apache::AxKit::Plugin::BasicSession::session{authlevel} < 4)) {
+      if (! defined($authlevel)
+	  || ($authlevel < 4)) {
 	$limit{'editorok'} = 1;
       } elsif ($self->{editor}) {
 	$limit{'editorok'} = 0;
@@ -209,8 +189,8 @@ sub get_dom {
     my $anyarticles = 0;
     if($self->{articles}) {
       my %limit;
-      if (! defined($Apache::AxKit::Plugin::BasicSession::session{authlevel})
-	  || ($Apache::AxKit::Plugin::BasicSession::session{authlevel} < 4)) {
+      if (! defined($authlevel)
+	  || ($authlevel < 4)) {
 	$limit{'editorok'} = 1;
 	$limit{'authorok'} = 1;
       }
@@ -226,7 +206,7 @@ sub get_dom {
     warn Dumper($self->{articles});
     
     unless (($anyarticles) || (defined($self->{stories}))) {
-      if (defined($Apache::AxKit::Plugin::BasicSession::session{authlevel})) {
+      if (defined($authlevel)) {
 	throw Apache::AxKit::Exception::Retval(
 					       return_code => 403,
 					       -text => "Articles or stories exist, but are only accessible to editors.");
@@ -262,21 +242,30 @@ sub get_styles {
 
 =head1 URI USAGE
 
+This Provider doesn't place any constraints of what first part of the
+local part of the URI should be, but the XSLT that is included in the
+distribution does, and you should set this with e.g.
+
+    PerlAddVar AxParamExpr cats.prefix '"/cats/"'
+
+in the Apache config.
+
+After that, you may simply append the C<catname>s of categories, and
+the Provider will return all stories and articles classified in B<all>
+those categories. Since for stories, there is a only a primary
+categorization, they are only displayed when there is a single
+category in the path.
 
 
+=head1 BUGS/TODO
 
-=head1 TODO
-
-Write more documentation.
-
-
-
+I guess this documentation could be more verbose.
 
 
 =head1 SEE ALSO
 
-L<AxKit::App::TABOO::Data::Provider::News>,
-L<AxKit::App::TABOO::Data::Plurals::Stories>
+L<AxKit::App::TABOO::Data::Provider::NewsList>,
+L<AxKit::App::TABOO::Data::Plurals::Categories>
 
 
 =head1 FORMALITIES
