@@ -2,19 +2,22 @@ package AxKit::App::TABOO::XSP::Story;
 use 5.6.0;
 use strict;
 use warnings;
+use utf8;
 use Apache::AxKit::Language::XSP::SimpleTaglib;
 use Apache::AxKit::Exception;
 use AxKit;
+use AxKit::App::TABOO;
 use AxKit::App::TABOO::Data::Story;
-use Apache::AxKit::Plugin::BasicSession;
+use AxKit::App::TABOO::Data::Plurals::Stories;
+use Session;
 use Time::Piece ':override';
 use XML::LibXML;
-
+use IDNA::Punycode;
 
 
 use vars qw/$NS/;
 
-our $VERSION = '0.095';
+our $VERSION = '0.4';
 
 =head1 NAME
 
@@ -64,6 +67,51 @@ use constant DIRECTOR  => 7;
 use constant GURU      => 8;
 use constant GOD       => 9;
 
+# This sub will create a useful storyname from the title
+sub _create_storyname {
+    my $intitle = shift;
+    my $nl = 30; # Should be the length of storyname in the table
+    chomp($intitle);
+ 
+    my $endno = '';
+    $intitle =~ s/\p{TerminalPunctuation}//gs; # Remove terminal punctutation
+    if ($intitle =~ m/(\d+)$/) { # if it ends with a number, the number might be useful, so preserve it
+	$endno = $1;
+    }
+    idn_prefix('');
+#    warn "INTITLE: ". $intitle;
+    $intitle = encode_punycode(lc($intitle));
+    $intitle =~ s/\s/_/gs; # All spaces become _
+    $intitle =~ s/[^a-z0-9\-_]//g; # Remove all now not a alphanumeric or -
+#    warn $intitle;
+#    warn $endno;
+#    warn length($endno);
+    
+    my $base = substr($intitle, 0, $nl-2-length($endno));
+    
+#    warn $base;
+    my $storyname = $base . $endno;
+    my $stories = AxKit::App::TABOO::Data::Plurals::Stories->new();
+    
+    my $i = 1;    
+    while ($stories->exists(storyname => $storyname)) { # Checking if the story exists
+	$base = substr($base, 0, $nl-3-length($i)-length($endno));
+	$storyname = $base . $endno . '-' . $i;
+	AxKit::Debug(9, "Try '$storyname' for storyname");
+
+	$i++;
+	if ($i>99) { # We've gone too far allready
+	    throw Apache::AxKit::Exception::Retval(
+						   return_code => 500,
+						   -text => "Tried $i storynames, all taken. Editors need more imagination.");
+	}
+    }
+    if (length($storyname) > $nl) { # Is likely to cause a crash
+	AxKit::Debug(2, "Length of '$storyname' is higher than $nl");
+    }
+    return $storyname;
+}
+
 package AxKit::App::TABOO::XSP::Story::Handlers;
 
 =head1 Tag Reference
@@ -96,19 +144,20 @@ sub store : node({http://www.kjetil.kjernsmo.net/software/TABOO/NS/Story/Output}
       $args{$name} = $cgi->param($name);
     }
 
-    $args{'username'} = $Apache::AxKit::Plugin::BasicSession::session{credential_0};
+    my $session = AxKit::App::TABOO::session($r);
+    $args{'username'} = AxKit::App::TABOO::loggedin($session);
 
-    my $authlevel =  $Apache::AxKit::Plugin::BasicSession::session{authlevel};
-  AxKit::Debug(6, "Logged in as $args{'username'} at level $authlevel");
+    my $authlevel = AxKit::App::TABOO::authlevel($session); 
+    AxKit::Debug(6, "Logged in as $args{'username'} at level $authlevel");
     unless (defined($authlevel)) {
 	throw Apache::AxKit::Exception::Retval(
 					       return_code => AUTH_REQUIRED,
 					       -text => "Not authenticated and authorized with an authlevel");
     }
-    if (($args{'sectionid'} eq 'subqueue') && (! $args{'storyname'}))
-    {
-	$args{'storyname'} = int(rand(100000));
-    } elsif ($args{'sectionid'} ne 'subqueue') {
+    unless ($args{'storyname'}) {
+	$args{'storyname'} = AxKit::App::TABOO::XSP::Story::_create_storyname($args{'title'});
+    } 
+    if ($args{'sectionid'} ne 'subqueue') {
 	if ($authlevel < AxKit::App::TABOO::XSP::Story::EDITOR) {
 	    throw Apache::AxKit::Exception::Retval(
 						   return_code => FORBIDDEN,
@@ -128,12 +177,6 @@ sub store : node({http://www.kjetil.kjernsmo.net/software/TABOO/NS/Story/Output}
 
     my $story = AxKit::App::TABOO::Data::Story->new();
 
-    my $oldstorykey = undef;
-    if ($args{'auto-storyname'}) {
-	$oldstorykey = $args{'auto-storyname'};
-	delete $args{'auto-storyname'};
-    }
-
     my $timestamp = localtime;
     unless ($args{'timestamp'}) {
 	$args{'timestamp'} = $timestamp->datetime;
@@ -143,7 +186,7 @@ sub store : node({http://www.kjetil.kjernsmo.net/software/TABOO/NS/Story/Output}
     }
 
     $story->populate(\%args);
-    $story->save($oldstorykey);
+    $story->save;
     1;
 EOC
 }
@@ -159,18 +202,15 @@ useful for previewing a submission.
 
 sub this_story : struct {
     return << 'EOC'
-	my %args = map { $_ => $cgi->param($_) } $cgi->param;
- 
+    my %args = map { $_ => $cgi->param($_) } $cgi->param;
 
-    $args{'username'} = $Apache::AxKit::Plugin::BasicSession::session{credential_0};
+    $args{'username'} = AxKit::App::TABOO::loggedin(AxKit::App::TABOO::session($r));
 
-    
     unless ($args{'submitterid'}) {
       # If the submitterid is not set, we set it to the current username
 	$args{'submitterid'} = $args{'username'}
     }
     
-    
     my $timestamp = localtime;
     unless ($args{'timestamp'}) {
       $args{'timestamp'} = $timestamp->datetime;
@@ -208,8 +248,10 @@ sub get_story : struct attribOrChild(storyname,sectionid) {
       $args{$name} = $cgi->param($name);
     }
 
+    my $session = AxKit::App::TABOO::session($r);
+
     unless ($args{'username'}) {
-      $args{'username'} = $Apache::AxKit::Plugin::BasicSession::session{credential_0};
+      $args{'username'} = AxKit::App::TABOO::loggedin($session);
     }
 
 
@@ -221,7 +263,7 @@ sub get_story : struct attribOrChild(storyname,sectionid) {
     }
     $story->adduserinfo();
     unless ($story->editorok) {
-      if ($Apache::AxKit::Plugin::BasicSession::session{authlevel} < 4) {
+      if (AxKit::App::TABOO::authlevel($session) < 4) {
 	throw Apache::AxKit::Exception::Retval(
 					       return_code => 401,
 					       -text => "Authentication and higher priviliges required to load story");
@@ -237,20 +279,6 @@ EOC
 
 1;
 
-=head1 Quirks 
-
-There are a few things that I'm not sure how to handle that I've
-included in this release in an inelegant way. For example, if you want
-to update an old record with a new storyname (which is not unusual, if
-for example you don't like the storyname used by the submitter), then
-you need to include this somehow. For the time being, you must supply
-the storyname as a query parameter C<auto-storyname>, and the supplied
-C<submit.xsp> does this. It is then understood by the
-C<E<lt>storeE<gt>> tag, which does the right thing, but I feel that
-such a tag shouldn't really need to be aware of such things, from an
-aestetical POW, suggestion on how to do it differently are welcome.
-
-
 =head1 FORMALITIES
 
 See L<AxKit::App::TABOO>.
